import os

from django.db import models
from django.urls import reverse

from users.models import CustomUser


def categorieImage(instance, filename):
    upload_to = 'media/_image/'
    ext = filename.split('.')[-1]
    # recuperation du nom de l'image
    if instance.nom:
        filename = '{}.{}'.format(instance.nom, ext)
        return os.path.join(upload_to, filename)


class Categorie(models.Model):
    nom = models.CharField(max_length=250)
    image = models.ImageField(upload_to=categorieImage)
    couleur = models.CharField(max_length=30)

    class Meta:
        verbose_name_plural = "categories"

    def __str__(self) -> str:
        return self.nom

    def get_absolute_url(self):
        return reverse("blog:categorieDetails", kwargs={"id": self.id})

    def allItems(self):
        postsItems = Post.objects.filter(categories=self.id).count()
        return postsItems

    # @classmethod
    # def categorie_posts(cls, num_posts=5):
    #     return Post.objects.filter(categories=cls.id)[:num_posts]


def coverImage(instance, filename):
    upload_to = 'media/_image/'
    ext = filename.split('.')[-1]
    # recuperation du nom de l'image
    if instance.title:
        filename = '{}.{}'.format(instance.title, ext)
        return os.path.join(upload_to, filename)


class Post(models.Model):
    title = models.CharField(max_length=255)
    cover = models.ImageField(upload_to=coverImage)
    body = models.TextField()
    created_on = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)
    views = models.IntegerField(default=0)
    categories = models.ManyToManyField("Categorie", related_name="posts")
    author = models.ForeignKey(CustomUser, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("blog:postDetails", kwargs={"id": self.id})

    @classmethod
    def most_viewed_posts(cls, num_posts=5):
        return cls.objects.order_by('-views')[:num_posts]

    @classmethod
    def lastest_posts(cls, num_posts=5):
        return cls.objects.order_by('-created_on')[:num_posts]

    @classmethod
    def shuffle_posts(cls, num_posts=5):
        return cls.objects.order_by('?')[:num_posts]


class TrendPost(models.Model):
    content = models.CharField(max_length=100)

    def __str__(self):
        return f"{self.id} - {self.content}"


class Comment(models.Model):
    author = models.CharField(max_length=60)
    body = models.TextField()
    created_on = models.DateTimeField(auto_now_add=True)
    post = models.ForeignKey("Post", on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.author} on '{self.post}'"
